# Workshop: Long-read genome assembly and pangenome graph analysis
Swiss TPH, 5.11.2024

The aim of this workshop is to get familiar with the pangenome graph framework, which, given a set of accurate assemblies, allows a full characterization of genetic variation of any type in any region of the genome.

It consists of two basic modules where we create a graph, visulize it, and call variants from it; and five "biological inference" modules where the graph is used to various things:


**Basics**
*  [Create a graph and visualize it](#create-a-graph--visualize-it)
*  [Call variants from a graph](#call-variants-from-a-graph)

**Biological inference**
*  [Core genome alignment](#core-genome-alignment)
*  [Graph annotation & liftovers](#graph-annotation--liftovers)
*  [Insertion or deletion?](#categorize-svs-into-insertions-and-deletions)
*  [Identify gene conversion events](#identify-gene-conversion-events)
*  [Large unassembled duplications](#large-unassembled-duplications)
*  [Aligning sequences to a graph (to do)](#aligning-sequences-to-a-graph)


## Set up
This workshop is based on Jupyiter notebooks, from where we run Python and bash code. You can run the notebooks from the sciCORE open-on-demand platform (http://ood-ubuntu.scicore.unibas.ch/) or from a editor like Visual Studio Code. 

We are going to use [PGGB](https://pggb.readthedocs.io/en/latest/) for creating analyzing pangenome graphs, a tool set developed by the human pangenome consortium. See [Yang et al. 2023](https://doi.org/10.3389/fgene.2023.1225248) for an evaluation of PGGB with bacterial genomes. In addition, a few Python packages need to be installed, as explained below. Please try to do at least steps 1 and 2 below **before** the workshop, especially step 2 might take some time.


Step 1: clone the repository from gitlab.
```
git clone https://git.scicore.unibas.ch/TBRU/PacbioSnake
```


Step 2: Pull the PGGB container (2Gb, might take a while...)
```
cd PacbioSnake
singularity pull docker://ghcr.io/pangenome/pggb:latest
```


Step 3: Install python packages (pandas and biopython). Ideally we create a new conda environment, which can then be selected as the kernel for running the notebooks.
```
conda create -n genomegraphs pandas biopython
```


## The data
The data we will explore in this workshop are 19 genomes that have been assembled from PacBio HiFi reads by Venus for her PhD project. All should be *Mycobacterium tuberculosis* strains belonging to lineage 1, and we expect them to be fairly diverse because they have been selected from across the diverse L1 clade. These genomes have neither been published nor thoroughly analyzed, so genuine discoveries are possible during the workshop!

## Basics

### Create a graph & visualize it
Given a set of assemblies and some assembly metadata, we select assemblies, rename them, and write them to a single fasta file. This is the only mandatory input for PGGB. We also explore the effect of two key parameters of PGGB, the minimum pairwise identity between seeds (-p) and the seed length (-s). 

### Call variants from a graph
In this part, we obtain variants in classic vcf format from the graph, using an arbitrary reference assembly. A summary is created of the indels and SVs, and the complication of nested SVs is explored. 


## Biological inference

### Core genome alignment
To make sense of genetic variation we need a phylogenetic tree. Here we traverse the graph and extract SNPs in nodes that are s in single strains. hared by all assemblies (i.e. the core genome). These SNPs are used to create an alignment and to estimate a tree. 

### Graph annotation & liftovers
In this part we explore variants in genes and regions of interest, making use of the lift-over functionalities of PGGB. These allow to translate positions in one genome to any other genome in the graph. 

### Categorize SVs into insertions and deletions
To tell whether a sequence missing in genome A reflects a deletion in an ancestor of A or an insertion elsewhere requires information about this sequence in more than two strains. Here we assess the frequency of he structural variant and its presence/absence in an outgroup strain in order to distinguish insertions from deletions.  

### Identify gene conversion events
Gene conversion occurs through recombination between close paralogs and can result in heavily mutated genes. Gene conversion in the MTBC affects the functionally interesting PE/PPE gene families. Furthermore, treating SNPs introduced throug gene conversion as point mutations can bias various downstream analyses. Here we identify gene conversion events, which give themselves away as variant hotspots in single strains. 

### Large unassembled duplications
Large duplications and amplifications pose a challenge for long-read sequencing. But even if these regions may not be assembled properly, duplications can be identified and studied. 


### Aligning sequences to a graph
Work in progress...